#!/usr/bin/env python3
import os
import sys
import glob
import yaml
from jinja2 import Environment, FileSystemLoader
import datetime as dt

# General configuration
class Config:
    magick = '/usr/bin/convert'
    sizes = [600, 800, 1200, 1600]
    suffixes = ['_sm', '_med', '_lg', '_hg']
    quality = [85, 85, 80, 75]
    thumb_path = 'thumbs'

# Photo data for the template
class Photo:
    def __init__(self, photo):
        self.filename = photo
        base_name, ext = photo.rsplit('.', 1)
        self.thumbnail = Config.thumb_path + '/' + base_name + Config.suffixes[0] + '.' + ext
        self.alt = ''

def main(args):
    """ev15_gallery: builds a gallery page from a folder full of
    jpg and/or png images

    Usage: ev15_gallery [new | thumbnails | html]

    If run without options will generate a new gallery if gallery.txt
    does not already exist, and will generate thumbnails if they
    do not already exist

    If run with new will generate a new gallery in all cases
    If run with thumbnails will regenerate thumbnails
    If run with html will rebuild html"""
    
    if len(args) > 1:
        if args[1] == 'new':
            build()
        elif args[1] == 'thumbnails':
            rebuild_thumbs()
        elif args[1] == 'html':
            rebuild_html()
        else:
            print(main.__doc__)
    else:
        build()

def site_data():
    # Site info
    f = open('../index.yaml', 'r')
    site = yaml.load(f.read(), Loader=yaml.SafeLoader)
    f.close()
    return site

def gallery_data():
    f = open('gallery.txt', 'r')
    gallery = yaml.load(f.read(), Loader=yaml.SafeLoader)
    f.close()
    return gallery

        
def build():
    print ("Make new gallery")
    gallery = gallery_info()
    photos = get_photos()
    gallery['photos'] = photos
    f = open('gallery.txt', 'w')
    f.write(yaml.dump(gallery))
    f.close()
    site = site_data()
    build_thumbs(gallery)
    build_html(site, gallery)

def rebuild_thumbs():
    print ("Rebuilding thumbnails")
    gallery = gallery_data()
    build_thumbs(gallery)
    
def rebuild_html():
    print ("Rebuilding HTML")
    site = site_data()
    gallery = gallery_data()
    build_html(site, gallery)
    
# Create photo gallery info file from user input    
def gallery_info():
    print('Enter photo gallery data. Title cannot be blank, other data can.')
    gallery = dict()
    gallery['title'] = input('Title: ')
    gallery['description'] = input('Description: ')
    gallery['tags'] = input('Tags (comma separated): ').split(',')
    print(gallery['tags'])
    # Enter a proper date or keep trying
    while 1:
        try:
            pub_date = parse_date(input('Publish date (yyyy-mm-dd): '))
            break
        except ValueError:
            print("Please enter date in yyyy-mm-dd format or leave blank to use today")
            continue

    gallery['date_published'] = pub_date  
    return gallery

# Take a text date and convert it to a date object
def parse_date(d):
    if(d == ''):
        return dt.date.today()
    else:
        return dt.datetime.strptime(d,'%Y-%m-%d').date()    

# Read photos in current folder
# Set to JPG and PNG files 
def get_photos():
    photos = glob.glob('*.[JjPp][PpNn][Gg]') #Matches any case of JPG or PNG
    photos.sort()
    return photos

# Build thumbnails
def build_thumbs(gallery):
    # Make thumbnails folder if it doesn't already exist
    if not(os.path.exists(Config.thumb_path)):
        os.mkdir(Config.thumb_path)

    # Build thumbnails if they do not exist
    # Thumbnail names and sizes are based on the configuration above
    # Also build other gallery information for the jinja template
    for photo in gallery['photos']:
        base_name, ext = photo.rsplit('.', 1)
        for i in range(0,len(Config.sizes)):
            output = Config.thumb_path + '/' + base_name + Config.suffixes[i] + '.' + ext
            if not(os.path.exists(output)):
                print("Building thumbnail {} for image {}".format(output, photo))
                os.system("{} {} -resize {}x{} -quality {} {}".format(Config.magick, photo, Config.sizes[i], Config.sizes[i], Config.quality[i], output))
            else:
                print("Thumbnail {} for image {} already exists".format(output, photo))

# Make html file
def build_html(site, gallery):
    photos = []
    for photo in gallery['photos']:
        photos.append(Photo(photo))

    # Set up jinja templates
    env = Environment(
        loader=FileSystemLoader('../templates')
    )
    try:
        template = env.get_template('gallery.html')
    except TemplateSyntaxError:
        print("Problem with the template")
    
    # Write index.html
    f = open('index.html', 'w')
    f.write(template.render(name = site['name'],
                            title = site['title'],
                            gallery = gallery, 
                            photos = photos))
    f.close()

if __name__ == "__main__":
    main(sys.argv)